@@ -112,8 +112,6 @@ static void hd_switcher_item_closed (HdSwitcher *switcher,
 
 static void hd_switcher_group_background_clicked (HdSwitcher   *switcher,
 						  ClutterActor *actor);
-static void hd_switcher_home_background_clicked (HdSwitcher   *switcher,
-						 ClutterActor *actor);
 
 static gboolean hd_switcher_notification_clicked (HdSwitcher *switcher,
                                                   HdNote *note);
@@ -188,9 +186,6 @@ hd_switcher_constructed (GObject *object)
 {
   GError            *error = NULL;
   HdSwitcherPrivate *priv = HD_SWITCHER (object)->priv;
-  HdHome	    *home =
-    HD_HOME (hd_comp_mgr_get_home (HD_COMP_MGR (priv->comp_mgr)));
-
 
   g_signal_connect_swapped (hd_render_manager_get_title_bar(),
                             "clicked-top-left",
@@ -204,9 +199,6 @@ hd_switcher_constructed (GObject *object)
                               "leave-top-left",
                               G_CALLBACK (hd_switcher_leave),
                               object);
-  g_signal_connect_swapped (home, "background-clicked",
-                            G_CALLBACK (hd_switcher_home_background_clicked),
-                            object);
   /* Task Launcher events */
   priv->launcher = hd_launcher_get ();
   g_signal_connect (priv->launcher, "launcher-hidden",
@@ -455,8 +447,6 @@ hd_switcher_leave (HdSwitcher *switcher)
 {
   HdSwitcherPrivate *priv = switcher->priv;
 
-  priv->pressed = FALSE;
-
   if (priv->press_timeout)
     {
       g_source_remove (priv->press_timeout);
@@ -591,9 +581,11 @@ hd_switcher_loading_fail (HdSwitcher *switcher,
         hd_render_manager_set_state (HDRM_STATE_HOME);
     }
 
+#if 0 /* removed as of NB#140674 */
   GtkWidget* banner = hildon_banner_show_information (NULL, NULL,
                         _("ckct_ib_application_loading_failed"));
   hildon_banner_set_timeout (HILDON_BANNER (banner), 6000);
+#endif
 }
 
 /*
@@ -748,7 +740,7 @@ hd_switcher_zoom_in_complete (ClutterActor *actor, HdSwitcher *switcher)
       hd_render_manager_set_loading (actor);
       hd_render_manager_set_state (HDRM_STATE_LOADING);
       hd_render_manager_stop_transition ();
-      hd_title_bar_set_title (tbar, text, FALSE, TRUE);
+      hd_title_bar_set_loading_title (tbar, text);
 
       /*
        * Implementing a timeout to see if the wakeup fails.
@@ -923,21 +915,6 @@ hd_switcher_hibernate_window_actor (HdSwitcher   * switcher,
   hd_task_navigator_hibernate_window (priv->task_nav, actor);
 }
 
-ClutterActor *
-hd_switcher_get_task_navigator (HdSwitcher *switcher)
-{
-  HdSwitcherPrivate *priv = HD_SWITCHER (switcher)->priv;
-  return CLUTTER_ACTOR(priv->task_nav);
-}
-
-static void
-hd_switcher_home_background_clicked (HdSwitcher   *switcher,
-	   			         ClutterActor *actor)
-{
-  g_debug("hd_switcher_home_background_clicked: switcher=%p\n", switcher);
-  hd_switcher_group_background_clicked (switcher, actor);
-}
-
 static void
 hd_switcher_group_background_clicked (HdSwitcher   *switcher,
 				      ClutterActor *actor)
