@@ -43,6 +43,8 @@
 #include "hd-launcher.h"
 #include "hd-launcher-item.h"
 #include "hd-comp-mgr.h"
+#include "hd-util.h"
+#include "hd-transition.h"
 
 #define I_(str) (g_intern_static_string ((str)))
 
@@ -74,6 +76,15 @@ struct _HdLauncherGridPrivate
 
   TidyAdjustment *h_adjustment;
   TidyAdjustment *v_adjustment;
+
+  /* How far the transition will move the icons */
+  gint transition_depth;
+  /* Do we move the icons all together or in sequence? for launcher_in transitions */
+  gboolean transition_sequenced;
+  /* List of keyframes used on transitions like _IN and _IN_SUB */
+  HdKeyFrameList *transition_keyframes; // ramp for tile movement
+  HdKeyFrameList *transition_keyframes_label; // ramp for label alpha values
+  HdKeyFrameList *transition_keyframes_icon; // ramp for icon alpha values
 };
 
 enum
@@ -93,7 +104,6 @@ enum
 static guint task_signals[LAST_SIGNAL] = {};
 */
 
-static void clutter_container_iface_init (ClutterContainerIface   *iface);
 static void tidy_scrollable_iface_init   (TidyScrollableInterface *iface);
 
 static gboolean _hd_launcher_grid_blocker_release_cb (ClutterActor *actor,
@@ -104,9 +114,7 @@ static gboolean _hd_launcher_grid_blocker_release_cb (ClutterActor *actor,
 
 G_DEFINE_TYPE_WITH_CODE (HdLauncherGrid,
                          hd_launcher_grid,
-                         CLUTTER_TYPE_ACTOR,
-                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
-                                                clutter_container_iface_init)
+                         CLUTTER_TYPE_GROUP,
                          G_IMPLEMENT_INTERFACE (TIDY_TYPE_SCROLLABLE,
                                                 tidy_scrollable_iface_init));
 
@@ -164,9 +172,20 @@ hd_launcher_grid_refresh_v_adjustment (HdLauncherGrid *grid)
   clutter_actor_get_clipu (CLUTTER_ACTOR (grid),
                            NULL, &clip_y,
                            NULL, &clip_height);
+  if (height >= CLUTTER_UNITS_FROM_INT (HD_COMP_MGR_LANDSCAPE_HEIGHT))
+    {
+      /* Padding at the bottom. */
+      height += CLUTTER_UNITS_FROM_INT (HD_LAUNCHER_BOTTOM_MARGIN
+                                      - HD_LAUNCHER_GRID_ROW_SPACING);
+      tidy_adjustment_set_skirtx (priv->v_adjustment,
+                     clutter_qdivx (CFX_ONE, CLUTTER_INT_TO_FIXED (4)));
+    }
+  else
+    tidy_adjustment_set_skirtx (priv->v_adjustment, 0);
 
   if (clip_height == 0)
-    page_height = CLUTTER_UNITS_TO_FIXED (height);
+    page_height = MIN (CLUTTER_UNITS_TO_FIXED (height),
+                       CLUTTER_UNITS_TO_FIXED (CLUTTER_UNITS_FROM_DEVICE (HD_COMP_MGR_LANDSCAPE_HEIGHT)));
   else
     page_height = MIN (CLUTTER_UNITS_TO_FIXED (height),
                        CLUTTER_UNITS_TO_FIXED (clip_height - clip_y));
@@ -194,9 +213,11 @@ adjustment_value_notify (GObject    *gobject,
                          gpointer    user_data)
 {
   ClutterActor *grid = user_data;
+  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (grid)->priv;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (grid))
-    clutter_actor_queue_redraw (grid);
+  clutter_actor_set_anchor_point(grid,
+                             0,
+                             tidy_adjustment_get_value(priv->v_adjustment));
 }
 
 static void
@@ -266,6 +287,7 @@ hd_launcher_grid_get_adjustments (TidyScrollable  *scrollable,
           hd_launcher_grid_set_adjustments (scrollable,
                                             adjustment,
                                             priv->v_adjustment);
+          g_object_unref (adjustment);
           hd_launcher_grid_refresh_h_adjustment (HD_LAUNCHER_GRID (scrollable));
 
           *h_adj = adjustment;
@@ -284,6 +306,7 @@ hd_launcher_grid_get_adjustments (TidyScrollable  *scrollable,
           hd_launcher_grid_set_adjustments (scrollable,
                                             priv->h_adjustment,
                                             adjustment);
+          g_object_unref (adjustment);
           hd_launcher_grid_refresh_v_adjustment (HD_LAUNCHER_GRID (scrollable));
 
           *v_adj = adjustment;
@@ -299,69 +322,40 @@ tidy_scrollable_iface_init (TidyScrollableInterface *iface)
 }
 
 static void
-hd_launcher_grid_add (ClutterContainer *container,
-                      ClutterActor     *actor)
+hd_launcher_grid_actor_added (ClutterContainer *container,
+                              ClutterActor     *actor)
 {
   HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (container)->priv;
 
   g_object_ref (actor);
 
-  priv->tiles = g_list_append (priv->tiles, actor);
-  clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
-
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
-
-  if (priv->h_adjustment)
-    hd_launcher_grid_refresh_h_adjustment (HD_LAUNCHER_GRID (container));
-
-  if (priv->v_adjustment)
-    hd_launcher_grid_refresh_v_adjustment (HD_LAUNCHER_GRID (container));
-
-  g_signal_emit_by_name (container, "actor-added", actor);
-
-  g_object_unref (actor);
-}
-
-static void
-hd_launcher_grid_remove (ClutterContainer *container,
-                         ClutterActor     *actor)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (container)->priv;
-
-  g_object_ref (actor);
-
-  priv->tiles = g_list_remove (priv->tiles, actor);
-  clutter_actor_unparent (actor);
-
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
-
-  g_signal_emit_by_name (container, "actor-removed", actor);
-
-  g_object_unref (actor);
-}
-
-static void
-hd_launcher_grid_foreach (ClutterContainer *container,
-                          ClutterCallback   callback,
-                          gpointer          callback_data)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (container)->priv;
-  GList *l;
-
-  for (l = priv->tiles; l != NULL; l = l->next)
+  if (HD_IS_LAUNCHER_TILE(actor))
     {
-      ClutterActor *child = l->data;
+      priv->tiles = g_list_append (priv->tiles, g_object_ref(actor));
 
-      callback (child, callback_data);
+      /* relayout moved to the traversal code */
     }
+
+  g_object_unref (actor);
 }
 
 static void
-clutter_container_iface_init (ClutterContainerIface *iface)
+hd_launcher_grid_actor_removed (ClutterContainer *container,
+                                ClutterActor     *actor)
 {
-  iface->add = hd_launcher_grid_add;
-  iface->remove = hd_launcher_grid_remove;
-  iface->foreach = hd_launcher_grid_foreach;
+  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (container)->priv;
+
+  g_object_ref (actor);
+
+  if (HD_IS_LAUNCHER_TILE(actor))
+    {
+      priv->tiles = g_list_remove (priv->tiles, actor);
+      g_object_unref(actor);
+
+      /* relayout moved to the traversal code */
+    }
+
+  g_object_unref (actor);
 }
 
 static void
@@ -390,44 +384,18 @@ _hd_launcher_grid_count_children_and_rows (HdLauncherGrid *grid,
     *rows = 0;
 }
 
-static void
-hd_launcher_grid_get_preferred_height (ClutterActor *actor,
-                                       ClutterUnit   for_width,
-                                       ClutterUnit  *min_height_p,
-                                       ClutterUnit  *natural_height_p)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID_GET_PRIVATE (actor);
-  guint n_visible_launchers, n_rows, natural_height;
-
-  if (min_height_p)
-    *min_height_p = CLUTTER_UNITS_FROM_DEVICE (HD_LAUNCHER_GRID_MIN_HEIGHT);
-
-  if (natural_height_p)
-    {
-      _hd_launcher_grid_count_children_and_rows (HD_LAUNCHER_GRID (actor),
-          &n_visible_launchers, &n_rows);
-
-        natural_height = HD_LAUNCHER_PAGE_YMARGIN +
-                         HD_LAUNCHER_TILE_HEIGHT * n_rows +
-                         (n_rows > 0 ? priv->v_spacing * (n_rows - 1) : 0);
-        *natural_height_p = CLUTTER_UNITS_FROM_DEVICE (MAX (natural_height,
-                                HD_LAUNCHER_GRID_MIN_HEIGHT));
-    }
-}
 
 /**
  * Allocates a number of tiles in a row, starting at cur_y.
  * Returns the number of children allocated.
  */
 static GList *
-_hd_launcher_grid_allocate_row (GList *l,
+_hd_launcher_grid_layout_row   (GList *l,
                                 guint *remaining,
                                 guint cur_y,
-                                guint h_spacing,
-                                gboolean origin_changed)
+                                guint h_spacing)
 {
   ClutterActor *child;
-  ClutterActorBox box;
   guint allocated = MIN (HD_LAUNCHER_GRID_MAX_COLUMNS, *remaining);
   /* Figure out the starting X position needed to centre the icons */
   guint icons_width = HD_LAUNCHER_TILE_WIDTH * HD_LAUNCHER_GRID_MAX_COLUMNS +
@@ -438,169 +406,78 @@ _hd_launcher_grid_allocate_row (GList *l,
     {
       child = l->data;
 
-      box.x1 = CLUTTER_UNITS_FROM_DEVICE (cur_x);
-      box.y1 = CLUTTER_UNITS_FROM_DEVICE (cur_y);
-      cur_x += HD_LAUNCHER_TILE_WIDTH;
-      box.x2 = CLUTTER_UNITS_FROM_DEVICE (cur_x);
-      box.y2 = CLUTTER_UNITS_FROM_DEVICE (cur_y + HD_LAUNCHER_TILE_HEIGHT);
-      clutter_actor_allocate (child, &box, origin_changed);
+      clutter_actor_set_position(child, cur_x, cur_y);
+      cur_x += HD_LAUNCHER_TILE_WIDTH + h_spacing;
 
-      cur_x += h_spacing;
       l = l->next;
     }
   *remaining -= allocated;
   return l;
 }
 
-static void
-hd_launcher_grid_allocate (ClutterActor          *actor,
-                           const ClutterActorBox *box,
-                           gboolean               origin_changed)
+void hd_launcher_grid_layout (HdLauncherGrid *grid)
 {
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (actor)->priv;
-  ClutterActorClass *parent_class;
+  HdLauncherGridPrivate *priv = grid->priv;
   GList *l;
   guint cur_height, n_visible_launchers, n_rows;
 
-  /* Free our list of 'blocker' actors that we use to block mouse clicks */
+  /* Free our list of 'blocker' actors that we use to block mouse clicks.
+   * TODO: just check we have 'nrows' worth */
   g_list_foreach(priv->blockers,
-                 (GFunc) clutter_actor_destroy,
+                 (GFunc)clutter_actor_destroy,
                  NULL);
   g_list_free(priv->blockers);
   priv->blockers = NULL;
 
-  /* chain up to save the allocation */
-  parent_class = CLUTTER_ACTOR_CLASS (hd_launcher_grid_parent_class);
-  parent_class->allocate (actor, box, origin_changed);
-
-  _hd_launcher_grid_count_children_and_rows (HD_LAUNCHER_GRID (actor),
+  _hd_launcher_grid_count_children_and_rows (grid,
       &n_visible_launchers, &n_rows);
 
   cur_height = HD_LAUNCHER_PAGE_YMARGIN;
+
+  /* This actually does the scrolling, instead of us using
+   * cogl_translate like we did previously, which kills software
+   * selection mode */
+  cur_height -= tidy_adjustment_get_value(priv->v_adjustment);
+
   l = priv->tiles;
   while (l) {
     /* Allocate all icons on this row */
-    l = _hd_launcher_grid_allocate_row(l, &n_visible_launchers,
-                                       cur_height, priv->h_spacing,
-                                       origin_changed);
+    l = _hd_launcher_grid_layout_row(l, &n_visible_launchers,
+                                       cur_height, priv->h_spacing);
     if (l)
       {
         /* If there is another row, we must create an actor that
          * goes between the two rows that will grab the clicks that
          * would have gone between them and dismissed the launcher  */
-        ClutterActorBox box;
         ClutterActor *blocker = clutter_group_new();
         clutter_actor_set_name(blocker, "HdLauncherGrid::blocker");
         clutter_actor_show(blocker);
-        clutter_actor_set_parent (blocker, actor);
+        clutter_container_add_actor(CLUTTER_CONTAINER(grid), blocker);
         clutter_actor_set_reactive(blocker, TRUE);
         g_signal_connect (blocker, "button-release-event",
                           G_CALLBACK (_hd_launcher_grid_blocker_release_cb),
                           NULL);
-        box.x1 = CLUTTER_UNITS_FROM_INT(HD_LAUNCHER_LEFT_MARGIN);
-        box.y1 = CLUTTER_UNITS_FROM_INT(cur_height + HD_LAUNCHER_TILE_HEIGHT);
-        box.x2 = CLUTTER_UNITS_FROM_INT(
-            HD_LAUNCHER_GRID_WIDTH - HD_LAUNCHER_RIGHT_MARGIN);
-        box.y2 = CLUTTER_UNITS_FROM_INT(cur_height +
-            HD_LAUNCHER_TILE_HEIGHT + priv->v_spacing);
-        clutter_actor_allocate(blocker, &box, origin_changed);
+        clutter_actor_set_position(blocker,
+                                   HD_LAUNCHER_LEFT_MARGIN,
+                                   cur_height + HD_LAUNCHER_TILE_HEIGHT);
+        clutter_actor_set_size(blocker,
+            HD_LAUNCHER_GRID_WIDTH - (HD_LAUNCHER_LEFT_MARGIN+HD_LAUNCHER_RIGHT_MARGIN),
+            priv->v_spacing);
 
         priv->blockers = g_list_prepend(priv->blockers, blocker);
       }
     cur_height += HD_LAUNCHER_TILE_HEIGHT + priv->v_spacing;
   }
-}
 
-static void
-hd_launcher_grid_paint (ClutterActor *actor)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (actor)->priv;
-  GList *l;
+  clutter_actor_set_size(CLUTTER_ACTOR(grid),
+                         HD_LAUNCHER_PAGE_WIDTH,
+                         cur_height);
 
-  if (!CLUTTER_ACTOR_IS_VISIBLE (actor))
-    return;
+  if (priv->h_adjustment)
+    hd_launcher_grid_refresh_h_adjustment (grid);
 
-  cogl_push_matrix ();
-
-  /* offset by the adjustment value */
   if (priv->v_adjustment)
-    {
-      ClutterFixed v_offset = tidy_adjustment_get_valuex (priv->v_adjustment);
-
-      cogl_translatex (0, v_offset * -1, 0);
-    }
-
-  for (l = priv->tiles; l != NULL; l = l->next)
-    {
-      ClutterActor *child = l->data;
-
-      if (CLUTTER_ACTOR_IS_VISIBLE (child))
-        clutter_actor_paint (child);
-    }
-
-  cogl_pop_matrix ();
-}
-
-static void
-hd_launcher_grid_pick (ClutterActor       *actor,
-                       const ClutterColor *pick_color)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (actor)->priv;
-  GList *l;
-
-  CLUTTER_ACTOR_CLASS (hd_launcher_grid_parent_class)->pick (actor, pick_color);
-
-  cogl_push_matrix ();
-
-  /* offset by the adjustment value */
-  if (priv->v_adjustment)
-    {
-      ClutterFixed v_offset = tidy_adjustment_get_valuex (priv->v_adjustment);
-
-      cogl_translatex (0, v_offset * -1, 0);
-    }
-
-  for (l = priv->tiles; l != NULL; l = l->next)
-    {
-      ClutterActor *child = l->data;
-
-      if (CLUTTER_ACTOR_IS_VISIBLE (child))
-        clutter_actor_paint (child);
-    }
-  /* render blocking areas to stop clicks through to the background */
-  for (l = priv->blockers; l != NULL; l = l->next)
-    {
-      ClutterActor *child = l->data;
-
-      if (CLUTTER_ACTOR_IS_VISIBLE (child))
-        clutter_actor_paint (child);
-    }
-
-  cogl_pop_matrix ();
-}
-
-static void
-hd_launcher_grid_realize (ClutterActor *actor)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (actor)->priv;
-
-  g_list_foreach (priv->tiles,
-                  (GFunc) clutter_actor_realize,
-                  NULL);
-
-  CLUTTER_ACTOR_SET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-}
-
-static void
-hd_launcher_grid_unrealize (ClutterActor *actor)
-{
-  HdLauncherGridPrivate *priv = HD_LAUNCHER_GRID (actor)->priv;
-
-  CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-
-  g_list_foreach (priv->tiles,
-                  (GFunc) clutter_actor_unrealize,
-                  NULL);
+    hd_launcher_grid_refresh_v_adjustment (grid);
 }
 
 static void
@@ -614,9 +491,6 @@ hd_launcher_grid_dispose (GObject *gobject)
   g_list_free (priv->tiles);
   priv->tiles = NULL;
 
-  g_list_foreach(priv->blockers,
-                 (GFunc) clutter_actor_destroy,
-                 NULL);
   g_list_free(priv->blockers);
   priv->blockers = NULL;
 
@@ -691,7 +565,6 @@ static void
 hd_launcher_grid_class_init (HdLauncherGridClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (HdLauncherGridPrivate));
 
@@ -699,13 +572,6 @@ hd_launcher_grid_class_init (HdLauncherGridClass *klass)
   gobject_class->get_property = hd_launcher_grid_get_property;
   gobject_class->dispose = hd_launcher_grid_dispose;
 
-  actor_class->get_preferred_height = hd_launcher_grid_get_preferred_height;
-  actor_class->allocate = hd_launcher_grid_allocate;
-  actor_class->realize = hd_launcher_grid_realize;
-  actor_class->unrealize = hd_launcher_grid_unrealize;
-  actor_class->paint = hd_launcher_grid_paint;
-  actor_class->pick = hd_launcher_grid_pick;
-
   g_object_class_override_property (gobject_class,
                                     PROP_H_ADJUSTMENT,
                                     "hadjustment");
@@ -726,6 +592,11 @@ hd_launcher_grid_init (HdLauncherGrid *launcher)
   priv->v_spacing = HD_LAUNCHER_GRID_ROW_SPACING;
 
   clutter_actor_set_reactive (CLUTTER_ACTOR (launcher), FALSE);
+
+  g_signal_connect(
+      launcher, "actor-added", G_CALLBACK(hd_launcher_grid_actor_added), 0);
+  g_signal_connect(
+      launcher, "actor-removed", G_CALLBACK(hd_launcher_grid_actor_removed), 0);
 }
 
 ClutterActor *
@@ -755,14 +626,6 @@ hd_launcher_grid_clear (HdLauncherGrid *grid)
     }
 }
 
-static float sexy_overshoot(float x)
-{
-  float smooth_ramp, converge;
-  smooth_ramp = 1.0f - cos(x*3.141592);
-  converge = sin(0.5*3.141592*(1-x));
-  return (smooth_ramp*1.1)*converge + (1-converge);
-}
-
 /* Reset the grid before it is shown */
 void
 hd_launcher_grid_reset(HdLauncherGrid *grid)
@@ -784,11 +647,78 @@ hd_launcher_grid_reset(HdLauncherGrid *grid)
     }
 }
 
+
+void
+hd_launcher_grid_transition_begin(HdLauncherGrid *grid,
+                                  HdLauncherPageTransition trans_type)
+{
+  HdLauncherGridPrivate *priv = grid->priv;
+
+  priv->transition_depth =
+      hd_transition_get_int(
+                hd_launcher_page_get_transition_string(trans_type),
+                "depth",
+                100 /* default value */);
+  priv->transition_sequenced = 0;
+  if (trans_type == HD_LAUNCHER_PAGE_TRANSITION_IN ||
+      trans_type == HD_LAUNCHER_PAGE_TRANSITION_IN_SUB)
+    {
+      priv->transition_sequenced =
+            hd_transition_get_int(
+                      hd_launcher_page_get_transition_string(trans_type),
+                      "sequenced",
+                      0);
+      if (priv->transition_sequenced)
+        {
+          grid->priv->transition_keyframes =
+            hd_transition_get_keyframes(
+                      hd_launcher_page_get_transition_string(trans_type),
+                      "keyframes", "0,1");
+          grid->priv->transition_keyframes_label =
+            hd_transition_get_keyframes(
+                      hd_launcher_page_get_transition_string(trans_type),
+                      "keyframes_label", "0,1");
+          grid->priv->transition_keyframes_icon =
+                 hd_transition_get_keyframes(
+                      hd_launcher_page_get_transition_string(trans_type),
+                      "keyframes_icon", "0,1");
+        }
+
+      /* Reset adjustments so the view is always back to 0,0 */
+      if (priv->h_adjustment)
+        tidy_adjustment_set_valuex (priv->h_adjustment, 0);
+
+      if (priv->v_adjustment)
+        tidy_adjustment_set_valuex (priv->v_adjustment, 0);
+    }
+}
+
+void
+hd_launcher_grid_transition_end(HdLauncherGrid *grid)
+{
+  /* Free anything we may have allocated for the transition here */
+  if (grid->priv->transition_keyframes)
+    {
+      hd_key_frame_list_free(grid->priv->transition_keyframes);
+      grid->priv->transition_keyframes = 0;
+    }
+  if (grid->priv->transition_keyframes_label)
+    {
+      hd_key_frame_list_free(grid->priv->transition_keyframes_label);
+      grid->priv->transition_keyframes_label = 0;
+    }
+  if (grid->priv->transition_keyframes_icon)
+    {
+      hd_key_frame_list_free(grid->priv->transition_keyframes_icon);
+      grid->priv->transition_keyframes_icon = 0;
+    }
+}
+
+
 void
 hd_launcher_grid_transition(HdLauncherGrid *grid,
                             HdLauncherPage *page,
                             HdLauncherPageTransition trans_type,
-                            int transition_depth,
                             float amount)
 {
   HdLauncherGridPrivate *priv;
@@ -853,19 +783,36 @@ hd_launcher_grid_transition(HdLauncherGrid *grid,
             case HD_LAUNCHER_PAGE_TRANSITION_IN:
             case HD_LAUNCHER_PAGE_TRANSITION_IN_SUB:
               {
-                float label_amt = (((order_amt*26) - 16) / 10);
-                float tile_amt = ((order_amt*26) / 16);
+                float label_amt, icon_amt;
+
+                if (priv->transition_sequenced)
+                  {
+                    label_amt = hd_key_frame_interpolate(
+                          priv->transition_keyframes_label, order_amt);
+                    icon_amt = hd_key_frame_interpolate(
+                          priv->transition_keyframes_icon, order_amt);
+
+                    if (label_amt<0) label_amt=0;
+                    if (label_amt>1) label_amt=1;
+                    if (icon_amt<0) icon_amt = 0;
+                    if (icon_amt>1) icon_amt = 1;
+                    depth = CLUTTER_UNITS_FROM_FLOAT(
+                       priv->transition_depth *
+                       (1 - hd_key_frame_interpolate(priv->transition_keyframes,
+                                                     order_amt)));
+                  }
+                else
+                  {
+                    depth = CLUTTER_UNITS_FROM_FLOAT(
+                                        priv->transition_depth * (1 - amount));
+                    label_amt = amount;
+                    icon_amt = amount;
+                  }
 
-                if (tile_amt<0) tile_amt = 0;
-                if (tile_amt>1) tile_amt = 1;
-                if (label_amt<0) label_amt=0;
-                if (label_amt>1) label_amt=1;
-                depth = CLUTTER_UNITS_FROM_FLOAT(
-                    transition_depth * (1 - sexy_overshoot(tile_amt)));
                 clutter_actor_set_depthu(CLUTTER_ACTOR(tile), depth);
                 clutter_actor_set_opacity(CLUTTER_ACTOR(tile), 255);
                 if (tile_icon)
-                  clutter_actor_set_opacity(tile_icon, (int)(tile_amt*255));
+                  clutter_actor_set_opacity(tile_icon, (int)(icon_amt*255));
                 if (tile_label)
                   clutter_actor_set_opacity(tile_label, (int)(label_amt*255));
                 break;
@@ -873,7 +820,7 @@ hd_launcher_grid_transition(HdLauncherGrid *grid,
             case HD_LAUNCHER_PAGE_TRANSITION_OUT:
             case HD_LAUNCHER_PAGE_TRANSITION_OUT_SUB:
               {
-                depth = CLUTTER_UNITS_FROM_FLOAT(transition_depth*amount);
+                depth = CLUTTER_UNITS_FROM_FLOAT(priv->transition_depth*amount);
                 clutter_actor_set_depthu(CLUTTER_ACTOR(tile), depth);
                 clutter_actor_set_opacity(CLUTTER_ACTOR(tile), 255 - (int)(amount*255));
                 break;
@@ -883,7 +830,7 @@ hd_launcher_grid_transition(HdLauncherGrid *grid,
                 float tile_amt = amount*2 - d;
                 if (tile_amt<0) tile_amt = 0;
                 if (tile_amt>1) tile_amt = 1;
-                depth = CLUTTER_UNITS_FROM_FLOAT(-transition_depth*tile_amt);
+                depth = CLUTTER_UNITS_FROM_FLOAT(-priv->transition_depth*tile_amt);
                 clutter_actor_set_depthu(CLUTTER_ACTOR(tile), depth);
                 clutter_actor_set_opacity(CLUTTER_ACTOR(tile), 255 - (int)(amount*255));
                 break;
@@ -891,12 +838,12 @@ hd_launcher_grid_transition(HdLauncherGrid *grid,
             /* We do't do anything for these now because we just use blur on
              * the whole group */
             case HD_LAUNCHER_PAGE_TRANSITION_BACK:
-                depth = CLUTTER_UNITS_FROM_FLOAT(-transition_depth*amount);
+                depth = CLUTTER_UNITS_FROM_FLOAT(-priv->transition_depth*amount);
                 clutter_actor_set_depthu(CLUTTER_ACTOR(tile), depth);
                 clutter_actor_set_opacity(CLUTTER_ACTOR(tile), 255 - (int)(amount*255));
                 break;
             case HD_LAUNCHER_PAGE_TRANSITION_FORWARD:
-                depth = CLUTTER_UNITS_FROM_FLOAT(-transition_depth*(1-amount));
+                depth = CLUTTER_UNITS_FROM_FLOAT(-priv->transition_depth*(1-amount));
                 clutter_actor_set_depthu(CLUTTER_ACTOR(tile), depth);
                 clutter_actor_set_opacity(CLUTTER_ACTOR(tile), (int)(amount*255));
                 break;
