@@ -51,6 +51,7 @@
 #include "hd-volume-profile.h"
 #include "launcher/hd-app-mgr.h"
 #include "home/hd-render-manager.h"
+#include "hd-transition.h"
 
 #ifndef DISABLE_A11Y
 #include "hildon-desktop-a11y.h"
@@ -61,6 +62,8 @@ enum {
   KEY_ACTION_TOGGLE_NON_COMP_MODE,
   KEY_ACTION_TAKE_SCREENSHOT,
   KEY_ACTION_XTERMINAL,
+  KEY_ACTION_TOGGLE_PORTRAITABLE,
+  KEY_ACTION_ROTATE,
 };
 
 #ifdef MBWM_DEB_VERSION
@@ -123,7 +126,7 @@ take_screenshot (void)
 {
   char *path, *filename;
   static gchar datestamp[255];
-  time_t secs;
+  static time_t secs = 0;
   struct tm *tm = NULL;
   GdkDrawable *window;
   int width, height;
@@ -136,6 +139,11 @@ take_screenshot (void)
     return;
   }
 
+  /* limit the rate of screenshots to avoid jamming HD when the key
+   * is pressed all the time */
+  if (time (NULL) - secs < 5)
+    return;
+
   path = g_strdup_printf ("%s/.images/Screenshots", getenv("MYDOCSDIR"));
   g_mkdir_with_parents (path, 0770);
 
@@ -187,6 +195,8 @@ theme_client_type_func (const char *type_name, void *user_data)
     return HdWmClientTypeAppMenu;
   else if (!strcmp (type_name, "home-applet"))
     return HdWmClientTypeHomeApplet;
+  else if (!strcmp (type_name, "status-menu"))
+      return HdWmClientTypeStatusMenu;
 
   return 0;
 }
@@ -201,6 +211,31 @@ theme_button_type_func (const char *type_name,
   return 0;
 }
 
+/* Toggle the portrait-capable flag on and off on the topmost window */
+static void
+toggle_portraitable(MBWindowManager   *wm)
+{
+  MBWindowManagerClient *c;
+  for (c=wm->stack_top;c;c=c->stacked_below)
+    {
+      MBWMClientType c_type = MB_WM_CLIENT_CLIENT_TYPE (c);
+      if (c_type == MBWMClientTypeDesktop)
+        break;
+      if (c_type == MBWMClientTypeApp)
+        {
+          /* actually set the portrait property to the opposite now */
+          gboolean new_supports = !hd_comp_mgr_client_supports_portrait(c);
+          guint value = new_supports ? 1 : 0;
+          mb_wm_util_async_trap_x_errors (wm->xdpy);
+          XChangeProperty(wm->xdpy, c->window->xwindow,
+                          wm->atoms[MBWM_ATOM_HILDON_PORTRAIT_MODE_SUPPORT],
+                          XA_CARDINAL, 32, PropModeReplace,
+                          (unsigned char *)&value, 1);
+          mb_wm_util_async_untrap_x_errors ();
+        }
+    }
+}
+
 static void
 key_binding_func (MBWindowManager   *wm,
 		  MBWMKeyBinding    *binding,
@@ -213,31 +248,45 @@ key_binding_func (MBWindowManager   *wm,
   switch (action)
     {
     case KEY_ACTION_TOGGLE_SWITCHER:
-      /* printf(" ### KEY_ACTION_TOGGLE_SWITCHER ###\n"); */
-      hd_render_manager_set_state (HDRM_STATE_TASK_NAV);
+      /* don't go to the switcher if we are showing a system-modal */
+      if (!hd_wm_has_modal_blockers (hd_mb_wm))
+        hd_render_manager_set_state (HDRM_STATE_TASK_NAV);
       break;
     case KEY_ACTION_TOGGLE_NON_COMP_MODE:
       /* printf(" ### KEY_ACTION_TOGGLE_NON_COMP_MODE ###\n"); */
       if (hd_render_manager_get_state () == HDRM_STATE_NON_COMPOSITED)
         hd_render_manager_set_state (HDRM_STATE_APP);
-      else
-        hd_render_manager_set_state (HDRM_STATE_NON_COMPOSITED);
+      else if (hd_render_manager_get_state () == HDRM_STATE_NON_COMP_PORT)
+        hd_render_manager_set_state (HDRM_STATE_APP_PORTRAIT);
+      else if (hd_render_manager_get_state () == HDRM_STATE_APP)
+        {
+          hd_render_manager_set_state (HDRM_STATE_NON_COMPOSITED);
+          /* render manager does not unredirect non-fullscreen apps,
+           * so do it here */
+          hd_comp_mgr_unredirect_topmost_client (hd_mb_wm, TRUE);
+        }
+      else if (hd_render_manager_get_state () == HDRM_STATE_APP_PORTRAIT)
+        {
+          hd_render_manager_set_state (HDRM_STATE_NON_COMP_PORT);
+          hd_comp_mgr_unredirect_topmost_client (hd_mb_wm, TRUE);
+        }
       break;
     case KEY_ACTION_TAKE_SCREENSHOT:
         take_screenshot();
 	break;
     case KEY_ACTION_XTERMINAL:
       {
-	pid_t fork(void);
-	int execl(const char *path, const char *arg, ...);
-
-	pid_t havechild=fork();
-	if (!havechild)
-	  execl("/usr/bin/osso-xterm",
-		"/usr/bin/osso-xterm", NULL);
-	/* This is a hack - don't care about the kid */
+        GPid pid;
+        if (hd_app_mgr_execute ("/usr/bin/osso-xterm", &pid, TRUE))
+          g_spawn_close_pid (pid);
+        break;
       }
-      break;
+    case KEY_ACTION_TOGGLE_PORTRAITABLE:
+        toggle_portraitable(wm);
+        break;
+    case KEY_ACTION_ROTATE:
+        hd_transition_rotate_screen (wm, !hd_comp_mgr_is_portrait());
+        break;
     }
 }
 
@@ -584,6 +633,16 @@ main (int argc, char **argv)
 				    key_binding_func,
 				    NULL,
 				    (void*)KEY_ACTION_TAKE_SCREENSHOT);
+  mb_wm_keys_binding_add_with_spec (wm,
+                                      "<shift><ctrl>r",
+                                      key_binding_func,
+                                      NULL,
+                                      (void*)KEY_ACTION_TOGGLE_PORTRAITABLE);
+  mb_wm_keys_binding_add_with_spec (wm, /* mod5 == Fn */
+                                      "<shift><ctrl><mod5>l",
+                                      key_binding_func,
+                                      NULL,
+                                      (void*)KEY_ACTION_ROTATE);
 
   clutter_x11_add_filter (clutter_x11_event_filter, wm);
 
